'' =================================================================================================
''
''   File....... jm_mcp23008.spin
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2011-15 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 24 DEC 2015
''               -- modified to allow all addresses from single object
''               -- streamlined with wr_reg() and rd_reg()
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                       ' eeprom / i2c
  SCL = 28


con { MCP23008 regs }

  MCP23008 = %0100_000_0                                         ' I2C device ID for MCP23008

  IODIR    = $00                                                 ' command bytes 
  IPOL     = $01 
  GPINTEN  = $02 
  DEFVAL   = $03
  INTCON   = $04 
  IOCON    = $05
  GPPU     = $06 
  INTF     = $07
  INTCAP   = $08
  GPIO     = $09
  OLAT     = $0A


obj

  i2c : "jm_i2c"


pub null

  ' This is not a top-level object


pub start

'' Connect MCP23008 to EEPROM I2C buss

  startx(SCL, SDA)                                               ' connect to boot EE buss

  
pub startx(sclpin, sdapin)

'' Connect MCP23008 to I2C buss
'' -- sclpin and sdapin are I2C buss pins

  i2c.setupx(sclpin, sdapin)                                     ' connect to i2C


pub present(addr)

'' Returns true if device at address is connected

  return i2c.present(MCP23008 | (addr << 1)) 
  

pub wr_reg(reg, value, addr)

'' Write value to register in device at addr
'' -- see constants section for valid registers
'' -- see MCP23008 docs for details on register value
'' -- addr is device address, %000 to %111

  i2c.start                                                      ' start transaction
  i2c.write(MCP23008 | (addr << 1))                              ' write id
  i2c.write(reg)                                                 ' write reg #
  i2c.write(value)                                               ' write reg value
  i2c.stop                                                       ' end transaction
                                                                  
                                                                  
pub rd_reg(reg, addr) | id, value                                 
                                                                  
'' Read value from register in device at addr                     
'' -- see constants section for valid registers                   
'' -- see MCP23008 docs for details on register value             
'' -- addr is device address, %000 to %111                        
                                                                  
  id := MCP23008 | (addr << 1)                                   ' id for write
                                                                  
  i2c.start                                                      ' start transaction
  i2c.write(id)                                                  ' write id
  i2c.write(reg)                                                 ' write reg #
  i2c.start                                                      ' re-start     
  i2c.write(id | %1)                                             ' id for read
  value := i2c.read(i2c#NAK)                                     ' read value from reg
  i2c.stop                                                       ' end transaction
                                                                  
  return value                                                   ' return value to caller
                                                                  
                                                                  
con                                                               
                                                                  
  { ------------------------------- }
  {  Named methods for easy access  }  
  { ------------------------------- }
   
  
pub cfg_dir(dirbits, addr)

'' Configure GPIO pins
'' -- 0 bit = output, 1 bit = input
'' -- addr is device address, %000 to %111

  wr_reg(IODIR, dirbits, addr)


pub cfg_pol(polbits, addr)  

'' Configure GPIO input polarity
'' -- 0 bit = normal, 1 bit = inverted
'' -- addr is device address, %000 to %111

  wr_reg(IPOL, polbits, addr)


pub cfg_iocon(iocbits, addr) 

'' Configure IOCON bits
'' -- see documentation for details on iocon bits
'' -- addr is device address, %000 to %111

  wr_reg(IOCON, iocbits, addr)
  

pub cfg_pu(pubits, addr)

'' Configure GPIO pull-ups
'' -- 1 bit = pull-up pin via 100K
'' -- addr is device address, %000 to %111

  wr_reg(GPPU, pubits, addr)
  

pub wr_gpio(outbits, addr)

'' Write outbits to GPIO pins
'' -- only pins set as outputs affected
'' -- addr is device address, %000 to %111

  wr_reg(GPIO, outbits, addr)


pub rd_gpio(addr)

'' Read GPIO bits
'' -- addr is device address, %000 to %111

  return rd_reg(GPIO, addr)


dat { license }

{{

  Copyright (C) 2011-15 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}